#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>

#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "wmgeneral.h"

#include "wmfmixer.xpm"

#include "linux/soundcard.h"

  /***********/
 /* Defines */
/***********/

#define WMFMIXER_VERSION "1.0.0"
#define DEVICE "/dev/mixer"

  /**********************/
 /* External Variables */
/**********************/

extern	char **environ;

  /********************/
 /* Global Variables */
/********************/

int	wmfmixer_mask_width = 64;
int	wmfmixer_mask_height = 64;
char	wmfmixer_mask_bits[64*64];
int	Vol[4] = {-1, -1, -1, -1};
int	SliderOver = 0;
int	ScopePos = 0;
const int 	Device[] = {SOUND_MIXER_VOLUME, SOUND_MIXER_PCM, SOUND_MIXER_LINE, SOUND_MIXER_CD};


  /***********************/
 /* Function Prototypes */
/***********************/

void	MoveSlider(int, int);
void	WriteDigits(int);
void	SetVolume(int, int);
int	GetVolume(int);
void	RedrawDevice(int);
int	fix(int, int, int);
void	UpdateVolumes();
void    Scope();

  /********/
 /* Main */
/********/

void main(int argc, char *argv[]) 
{
	int		i,SliderPressed = -1;

	XEvent			Event;

	/* Open the display */
	createXBMfromXPM(wmfmixer_mask_bits, wmfmixer_master_xpm, wmfmixer_mask_width, wmfmixer_mask_height);

	openXwindow(argc, argv, wmfmixer_master_xpm, wmfmixer_mask_bits, wmfmixer_mask_width, wmfmixer_mask_height);

	/* Sliders regions*/
	for (i=0; i<4; i++) AddMouseRegion(i, 5, 20+i*11, 59, 27+i*11);
	UpdateVolumes();
	RedrawDevice(0);

	/* main loop */
	while (1) {
		UpdateVolumes();
		Scope();
		while (XPending(display)) {
			XNextEvent(display, &Event);
			switch (Event.type) {
			case Expose:
				RedrawWindow();
				break;
			case DestroyNotify:
				XCloseDisplay(display);
				exit(0);
				break;
			case ButtonPress:
				SliderPressed = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
				if (SliderPressed == -1) break;
				SetVolume(SliderPressed, Event.xbutton.x);
				RedrawWindow();
				break;
			case ButtonRelease:
				SliderPressed = -1;
				break;
			case MotionNotify:
				i = CheckMouseRegion(Event.xmotion.x, Event.xbutton.y);
				if (i == -1) break;
				if (SliderPressed != -1) 
				{
					SetVolume(SliderPressed, Event.xmotion.x);
					RedrawWindow();
					break;
				}
				if (i != SliderOver) RedrawDevice(i);
				SliderOver = i;
				RedrawWindow();
				break;	
			default:
				break;
			}
		}
		usleep(50000L);
	}
}
  /*******************************/
 /****       SetVolume       ****/
/*******************************/
void SetVolume(int Slider, int x){
	int fd = open (DEVICE, 0);	
	int Volume;          
          
	MoveSlider(Slider, x=fix(x, 5, 51));      
	Volume  = ((x - 5)*100)/(51 - 5);
	WriteDigits(Volume);

	Volume |= Volume << 8;

	ioctl(fd, MIXER_WRITE(Device[Slider]), &Volume);
	close(fd);
}

  /*******************************/
 /****       GetVolume       ****/
/*******************************/
int GetVolume(int Slider)
{
	int fd = open(DEVICE, 0);
	int Volume;
	ioctl(fd, MIXER_READ(Device[Slider]), &Volume);
	Volume = Volume >> 8;
	close(fd);
	return Volume;
}


  /*******************************/
 /****      MoveSlider       ****/
/*******************************/
void MoveSlider(int Slider, int x) {
	copyXPMArea(0, 74, 54, 7, 5, 20+Slider*11);
	copyXPMArea(54,74, 7, 7, x, 20+Slider*11);
}

  /*******************************/
 /****          fix          ****/
/*******************************/

int fix(int num, int min, int max)
{
	if (num<min) num = min;
	if (num>max) num = max;
	return num;
}

  /*******************************/
 /****     WriteDigits       ****/
/*******************************/

void WriteDigits(int Volume)
{
	if (Volume == 100)
	{ 
		copyXPMArea(8, 65, 1, 9, 47, 5);
		copyXPMArea(0, 65, 4, 9, 49, 5);
		copyXPMArea(0, 65, 4, 9, 54, 5);
		return;
	}
	else	copyXPMArea(46, 5, 1, 9, 47, 5);
	
	copyXPMArea(0+(Volume%10)*5, 65, 4, 9, 54, 5);
	
	if ((Volume /= 10) == 0)
		copyXPMArea(46, 5, 4, 9, 49, 5);
	else
		copyXPMArea(0+(Volume%10)*5, 65, 4, 9, 49, 5);
	
}

  /*******************************/
 /****    RedrawDevice       ****/
/*******************************/

void RedrawDevice(int DeviceName)
{
	copyXPMArea(0+DeviceName*16, 81, 15, 9, 27, 5);
	WriteDigits(GetVolume(DeviceName));
}

  /*******************************/
 /****    UpdateVolume       ****/
/*******************************/

void UpdateVolumes()
{
	int i, j;
	int changed = 0;
	for (i=0; i<4; i++)
	{
		if (Vol[i] != (j = GetVolume(i))) 
		{
			changed = 1;
			MoveSlider(i, 5+(j*(51-5))/100);
			if (SliderOver == i) WriteDigits(j);
		}
		Vol[i] = j;
	}	
	if (changed) RedrawWindow();
}

void Scope()
{
	copyXPMArea(7, 6, 20, 7, 6,6);
	copyXPMArea(ScopePos, 90, 1, 7, 22, 6);
	ScopePos++;
	if (ScopePos==64) ScopePos=0;
	RedrawWindow();
}